import requests, webbrowser, time, sys, random
from . import Utils, asset, errors
from typing import Type, Tuple
robloxpy = sys.modules['robloxpy']

user = None

class Place():
    __slots__ = ('id', 'name', 'description', 'url', 'builder', 'playable', 'universe', 'price')

    def __init__(self, id: int) -> None:
        self._update(id)

    def __repr__(self) -> str:
        return self.name

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Place):
            return self.id == other.id
        return False

    def _update(self, id: int) -> None:
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        data = robloxpy.CurrentCookie.get(f"{Utils.GamesAPI}games/multiget-place-details?placeIds={id}").json()[0]
        self.id = data['placeId']
        self.name = data['name']
        self.description = data['description']
        self.url = data['url']
        self.builder = user.User(data['builderId'])
        self.playable = data['isPlayable']
        self.universe = Universe(data['universeId'])
        self.price = data['price']

    def _icon(self, size: int = 512) -> str:
        return requests.get(f"{Utils.ThumnnailAPIV1}places/gameicons?format=Png&isCircular=false&placeIds={self.id}&size={size}x{size}").json()['data'][0]['imageUrl']

    @property
    def icon(self) -> Type[asset.ImageAsset]:
        """
        Returns the places icon as an ImageAsset.
        """
        return asset.ImageAsset(self._icon())
    
    def _thumbnail(self, width: int = 768, height: int = 432) -> str:
        if self.universe == None:
            raise errors.NoUniverse
        return requests.get(f"{Utils.ThumnnailAPIV1}games/multiget/thumbnails?format=Png&isCircular=false&universeIds={self.universe.id}&size={width}x{height}").json()['data'][0]['imageUrl']

    @property
    def thumbnail(self) -> Type[asset.ImageAsset]:
        """
        Returns the places thumbnail as an ImageAsset.
        """
        return asset.ImageAsset(self._thumbnail())

    @property
    def playing(self) -> int:
        """
        Returns the current ammount of players in the places universe.
        """
        if self.universe == None:
            raise errors.NoUniverse
        return self.universe.playing

    @property
    def visits(self) -> int:
        """
        Returns the total visits for the places universe.
        """
        if self.universe == None:
            raise errors.NoUniverse
        return self.universe.visits

    @property
    def favorites(self) -> int:
        """
        Returns the total favorites for the places universe.
        """
        if self.universe == None:
            raise errors.NoUniverse
        return self.universe.favorites

    @property
    def votes(self) -> int:
        """
        Returns the "score" of the places universe (upvotes - downvotes).
        """
        dat = self.raw_votes()
        return dat[0] - dat[1]

    def raw_votes(self) -> Tuple[int]:
        """
        Returns the raw vote data from the api. Returned as (upvotes, downvotes)
        """
        if self.universe == None:
            raise errors.NoUniverse
        return self.universe.raw_votes()

    def join_game(self) -> None:
        """
        Joins the given game
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        Gamesession = requests.session()
        Gamesession.cookies[".ROBLOSECURITY"] = robloxpy.RawCookie

        Gamesession = Gamesession.post(
        url = Utils.GameAuthUrl,
        headers = {
            "Referer": "https://www.roblox.com/",
            "X-CSRF-Token": Gamesession.post(
                url = Utils.GameAuthUrl
            ).headers["X-CSRF-Token"],
        }
        ).headers["RBX-Authentication-Ticket"]
        BrowserID = random.randint(10000000000, 99999999999)
        webbrowser.open(f"roblox-player:1+launchmode:play+gameinfo:{Gamesession}+launchtime:{int(time.time()*1000)}+placelauncherurl:https%3A%2F%2Fassetgame.roblox.com%2Fgame%2FPlaceLauncher.ashx%3Frequest%3DRequestGame%26browserTrackerId%3D{BrowserID}%26placeId%3D{self.id}%26isPlayTogetherGame%3Dfalse+browsertrackerid:{BrowserID}+robloxLocale:en_us+gameLocale:en_us")

class Universe():
    __slots__ = ('id', 'name', 'description', 'creator', 'price', 'allowed_gear_genres', 'genre_enforced', 'copying_allowed', 'max_players', 'created_at', 'updated_at', 'vip_servers', 'avatar_type', 'genre', 'all_genre', 'rating')
    
    def __init__(self, id: int) -> None:
        self._update(id)

    def __repr__(self) -> str:
        return self.name

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Universe):
            return self.id == other.id
        return False
        
    def _update(self, id: int) -> None:
        data = requests.get(f"{Utils.GamesAPI}games?universeIds={id}").json()['data'][0]
        self.id = data['id']
        self.name = data['name']
        self.description = data['description']
        self.creator = user.User(data['creator']['id'])
        self.price = data['price']
        self.allowed_gear_genres = data["allowedGearGenres"]
        self.genre_enforced = data['isGenreEnforced']
        self.copying_allowed = data['copyingAllowed']
        self.max_players = data['maxPlayers']
        self.created_at = data['created']
        self.updated_at = data['updated']
        self.vip_servers = data['createVipServersAllowed']
        self.avatar_type = data['universeAvatarType']
        self.genre = data['genre']
        self.all_genre = data['isAllGenre']
        self.rating = data['gameRating']

    @property
    def root_place(self) -> Type[Place]:
        """
        Returns the root place as a Place
        """
        return Place(requests.get(f"{Utils.GamesAPI}games?universeIds={self.id}").json()['data'][0]['rootPlaceId'])
    
    @property
    def playing(self) -> int:
        """
        Returns the current ammount of players in the universe
        """
        return requests.get(f"{Utils.GamesAPI}games?universeIds={self.id}").json()['data'][0]['playing']

    @property
    def visits(self) -> int:
        """
        Returns the total visits for the universe.
        """
        return requests.get(f"{Utils.GamesAPI}games?universeIds={self.id}").json()['data'][0]['visits']

    @property
    def favorites(self) -> int:
        """
        Returns the total favorites for the universe.
        """
        return requests.get(f"{Utils.GamesAPI}games?universeIds={self.id}").json()['data'][0]['favoritedCount']

    @property
    def votes(self) -> int:
        """
        Returns the "score" for the universe (upvotes - downvotes).
        """
        dat = self.raw_votes()
        return dat[0] - dat[1]

    def raw_votes(self) -> Tuple[int]:
        """
        Returns the raw vote data from the api. Returned as (upvotes, downvotes)
        """
        data = requests.get(f"{Utils.GamesAPI}games/votes?universeIds={self.id}").json()['data'][0]
        return (data['upVotes'], data['downVotes'])

    def _icon(self, size: int = 512) -> str:
        return requests.get(f"{Utils.ThumnnailAPIV1}games/icons?format=Png&isCircular=false&size={size}x{size}&universeIds={self.id}").json()['data'][0]['imageUrl']

    @property
    def icon(self) -> Type[asset.ImageAsset]:
        """
        Returns the universes icon as an ImageAsset
        """
        return asset.ImageAsset(self._icon())

    def join_game(self) -> None:
        """
        Joins the given game
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        Gamesession = requests.session()
        Gamesession.cookies[".ROBLOSECURITY"] = robloxpy.RawCookie

        Gamesession = Gamesession.post(
        url = Utils.GameAuthUrl,
        headers = {
            "Referer": "https://www.roblox.com/",
            "X-CSRF-Token": Gamesession.post(
                url = Utils.GameAuthUrl
            ).headers["X-CSRF-Token"],
        }
        ).headers["RBX-Authentication-Ticket"]
        BrowserID = random.randint(10000000000, 99999999999)
        webbrowser.open(f"roblox-player:1+launchmode:play+gameinfo:{Gamesession}+launchtime:{int(time.time()*1000)}+placelauncherurl:https%3A%2F%2Fassetgame.roblox.com%2Fgame%2FPlaceLauncher.ashx%3Frequest%3DRequestGame%26browserTrackerId%3D{BrowserID}%26placeId%3D{self.root_place.id}%26isPlayTogetherGame%3Dfalse+browsertrackerid:{BrowserID}+robloxLocale:en_us+gameLocale:en_us")

def _get_user():
    global user
    from . import user as usr
    user = usr
_get_user()


import requests, sys, webbrowser, time, random
from . import Utils, errors, asset, group
from typing import List, Tuple, Type, Union

robloxpy = sys.modules['robloxpy']
ClientId = None

class BaseUser():
    __slots__ = ('id', 'name', 'display_name', 'description', 'created_at', 'banned')

    def __init__(self, id: Union[int, str], fetch: bool = True):
        self._update(id, fetch)

    def __repr__(self) -> str:
        return self.name or str(self.id)

    def __eq__(self, other: object) -> bool:
        if isinstance(other, (BaseUser, PartialUser, User, ClientUser)):
            return self.id == other.id
        return False

    def _update(self, id: Union[int, str], fetch: bool) -> None:
        if type(id) != int:
            id = requests.get(Utils.APIURL + f'users/get-by-username?username={id}').json()['Id']
        if fetch:
            data = requests.get(f"{Utils.UserAPIV1}/{id}")
            try:
                rawData = data.json()
                self.id = id
                self.name = rawData['name']
                self.display_name = rawData['displayName']
                self.description = rawData['description']
                self.created_at = rawData['created']
                self.banned = rawData['isBanned']
            except (KeyError, AttributeError):
                raise errors.InvalidId
        else:
            self.id = id
            self.name = None
            self.display_name = None
            self.description = None
            self.created_at = None
            self.banned = None

    @property
    def online(self) -> bool:
        """
        Returns if the user is online.
        """
        data = requests.get(f"{Utils.UserAPI}{self.id}/onlinestatus")
        return data.json()['IsOnline']

    @property
    def name_history(self) -> List[str]:
        """
        Returns the users previous names.
        """
        Cursor = ""
        Done = False
        PastNames = []
        while(Done == False):
            response = requests.get(Utils.UserAPIV1 + f"{self.id}/username-history?limit=100&sortOrder=Asc&cursor={Cursor}")
            Names = response.json()['data']
            if((response.json()['nextPageCursor'] == "null") or response.json()['nextPageCursor'] == None):
                Done = True
            else:
                Done = False
                Cursor = response.json()['nextPageCursor']
            for Name in Names:
                PastNames.append(Name['name'])
            if(response.json()['nextPageCursor'] == 'None'):
                Done = True
        return PastNames

    @property
    def rap(self) -> int:
        """
        Returns the users recent average price.
        """
        TotalValue = 0
        Cursor = ""
        Done = False
        while(Done == False):
            try:
                response = requests.get(Utils.Inventory1URL + f"/{self.id}/assets/collectibles?sortOrder=Asc&limit=100&cursor={Cursor}")
                Items = response.json()
                if response.json()['nextPageCursor'] == "null" or response.json()['nextPageCursor'] == None:
                    Done = True
                else:
                    Done = False
                    Cursor = response.json()['nextPageCursor']
                for Item in Items["data"]:
                    try:
                        RAP = int(Item['recentAveragePrice'])
                        TotalValue = TotalValue + RAP
                    except:
                        TotalValue = TotalValue
                if response.json()['nextPageCursor'] == 'None':
                    Done = True
            except Exception as ex:
                Done = True
        return TotalValue

    @property
    def status(self) -> str:
        """
        Returns the users status.
        """
        response = requests.get(f"{Utils.UserAPIV1}{str(self.id)}/status")
        return response.json()['status']

    def _headshot(self, size: int = 720) -> str:
        response = requests.get(f"{Utils.ThumnnailAPIV1}users/avatar-headshot?userIds={self.id}&size={size}x{size}&format=png")
        return response.json()['data'][0]['imageUrl']

    @property
    def headshot(self) -> Type[asset.ImageAsset]:
        """
        Returns the users headshot as an ImageAsset.
        """
        return asset.ImageAsset(self._headshot())

    def _bust(self, size: int = 420) -> str:
        response = requests.get(f"{Utils.ThumnnailAPIV1}users/avatar-bust?userIds={self.id}&size={size}x{size}&format=png")
        return response.json()['data'][0]['imageUrl']

    @property
    def bust(self) -> Type[asset.ImageAsset]:
        """
        Returns the users bust as an ImageAsset.
        """
        return asset.ImageAsset(self._bust())

    def groups(self) -> Type[group.Group]:
        """
        Returns the groups the user is in
        """
        groups = []
        raw = requests.get(f"{Utils.GroupAPIV2}users/{self.id}/groups/roles")
        data = raw.json()
        for _group in data['data']:
            groups.append(group.Group(_group['group']['id']))
        return groups

    def limiteds(self) -> List[Type[asset.MarketAsset]]:
        """
        Returns the users limited items 
        """
        Limiteds = []
        Cursor = ""
        Done = False
        while(Done == False):
            try:
                response = requests.get(f"{Utils.InventoryURLV1}users/{self.id}/assets/collectibles?limit=100&sortOrder=Asc")
                Items = response.json()
                if response.json()['nextPageCursor'] == "null" or response.json()['nextPageCursor'] == None:
                    Done = True
                else:
                    Done = False
                    Cursor = response.json()['nextPageCursor']
                for Item in Items["data"]:
                    try:
                        Limiteds.append(asset.MarketAsset(Item['assetId']))
                    except:
                        continue
                if response.json()['nextPageCursor'] == 'None':
                    Done = True
                
            except Exception as ex:
                Done = True
        return Limiteds

class PartialUser():
    __slots__ = ('id')
    def __init__(self, id: int):
        self.id = id

    def __repr__(self) -> str:
        return str(self.id)

    def __eq__(self, other: object) -> bool:
        if isinstance(other, (BaseUser, PartialUser, User, ClientUser)):
            return self.id == other.id
        return False

    def fetch(self) -> 'User':
        """
        Returns the User instance for the PartialUser
        """
        return User(self.id)

class User(BaseUser):
    def __init__(self, id: Union[int, str]):
        super().__init__(id)
            
    @property
    def following(self) -> bool:
        """
        returns if the client user is following this user.
        redundant, might be removed
        """
        response = robloxpy.CurrentCookie.post(f"{Utils.FriendsAPI}user/following-exists?UserID={str(self.id)}&followerUserID={self.id}", data={'targetUserIDs': str(self.id)})
        return response.json()['followings'][0]['isFollowing']

    def send_message(self, Subject: str, Body: str) -> Union[str, dict]:
        """
        Sends the given message to the given user
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie

        response = None
        try:
            response = robloxpy.CurrentCookie.post(Utils.PrivateMessageAPIV1 + 'messages/send', data={
                                'userId': ClientId,
                                'subject': Subject,
                                'body': Body,
                                'recipientId': self.id,
                                })
            return response.json()
        except Exception as e:
            return response.json()

    def follow(self) ->  Union[bool, str]:
        """
        Follows a user.
        captcha blocked, might be removed.
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        response = robloxpy.CurrentCookie.post(f"{Utils.FriendsAPI}users/{self.id}/follow", data={'targetUserID': self.id})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']

    def unfollow(self) ->  Union[bool, str]:
        """
        unfollows the user
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        response = robloxpy.CurrentCookie.post(f"{Utils.FriendsAPI}users/{self.id}/unfollow", data={'targetUserID': self.id})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']

    def block(self) -> Union[bool, str]:
        """
        Blocks the user
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        response = robloxpy.CurrentCookie.post(f"{Utils.APIURL}userblock/block?userId={self.id}", data={'targetUserID': self.id})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']

    def unblock(self) -> Union[bool, str]:
        """
        Unlocks the user
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        response = robloxpy.CurrentCookie.post(f"{Utils.APIURL}userblock/unblock?userId={self.id}", data={'targetUserID': self.id})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']


class ClientUser(BaseUser):
    def __init__(self, id: Union[int, str]) -> None:
        super().__init__(id)
        global ClientId
        ClientId = self.id

    def blocked_users(self) -> List[Type[User]]:
        """
        Returns users which are blocked

        [UserID],[UserName]
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        response = robloxpy.CurrentCookie.get(f"{Utils.SettingsURL}")
        Data = response.json()['BlockedUsersModel']['BlockedUsers']
        BlockedUsers = []

        for _User in Data:
            BlockedUsers.append(User(_User['uid']))
        return BlockedUsers

    def following_user(self, targetUser: Union[Type[User], int]) -> bool:
        """
        Checks if the current account is following a user
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        if type(targetUser) == int:
            targetUser = User(targetUser)

        data = robloxpy.CurrentCookie.post(f"{Utils.FriendsAPI}user/following-exists?UserID={str(targetUser.id)}&followerUserID={self.id}", data={'targetUserIDs': str(targetUser.id)}).json()['followings'][0]
        return data['isFollowing']

    def join_game(self, PlaceId: int) -> None:
        """
        Joins the given game
        """
        if robloxpy.CurrentCookie == None:
            raise errors.NoCookie
        Gamesession = requests.session()
        Gamesession.cookies[".ROBLOSECURITY"] = robloxpy.RawCookie

        Gamesession = Gamesession.post(
        url = Utils.GameAuthUrl,
        headers = {
            "Referer": "https://www.roblox.com/",
            "X-CSRF-Token": Gamesession.post(
                url = Utils.GameAuthUrl
            ).headers["X-CSRF-Token"],
        }
        ).headers["RBX-Authentication-Ticket"]

        BrowserID = random.randint(10000000000, 99999999999)
        webbrowser.open(f"roblox-player:1+launchmode:play+gameinfo:{Gamesession}+launchtime:{int(time.time()*1000)}+placelauncherurl:https%3A%2F%2Fassetgame.roblox.com%2Fgame%2FPlaceLauncher.ashx%3Frequest%3DRequestGame%26browserTrackerId%3D{BrowserID}%26placeId%3D{PlaceId}%26isPlayTogetherGame%3Dfalse+browsertrackerid:{BrowserID}+robloxLocale:en_us+gameLocale:en_us")





import requests, os, sys
from json.decoder import JSONDecodeError

robloxpy = sys.modules['robloxpy']

TestCookie = ""

#URL List

MobileAPI = "https://www.roblox.com/mobileapi/"
FriendsAPI = "https://friends.roblox.com/v1/"
APIURL = "https://api.roblox.com/"
UserAPI = "https://api.roblox.com/users/"
UserAPIV1 = "https://users.roblox.com/v1/users/"
GroupAPIV1 = "https://groups.roblox.com/v1/groups/"
GroupAPIV2 = "https://groups.roblox.com/v2/"
EconomyURL = "https://economy.roblox.com/v1/"
EconomyURLV2 = "https://economy.roblox.com/v2/"
InventoryURLV1 = "https://inventory.roblox.com/v1/"
InventoryURLV2 = "https://inventory.roblox.com/v2/"
SettingsURL = "https://www.roblox.com/my/settings/json"
PrivateMessageAPIV1 = "https://privatemessages.roblox.com/v1/"
GamesAPI = "https://games.roblox.com/v1/"
DevelopAPIV2 = "https://develop.roblox.com/v2/"
GameAuthUrl = "https://auth.roblox.com/v1/authentication-ticket/"
ThumnnailAPIV1 = "https://thumbnails.roblox.com/v1/"

def SetProxy(ProxyIP: str) -> None:
    """
    Set the proxy to currently be used, this is global
    Format: IP:Port
    """
    #Format 144.217.101.245:3129
    if(ProxyIP != None):
        proxy = 'http://' + str(ProxyIP)
        os.environ['http_proxy'] = proxy
        os.environ['https_proxy'] = proxy

def CheckProxy(proxyAddress: str = None) -> str:
    """
    Check the current IP address being given by the program
    """
    SetProxy(proxyAddress)
    response = requests.get('https://api.ipify.org/?format=json')
    return response.json()['ip']

def CheckCookie(Cookie: str = None) -> bool:
    """
    If you want to check the current used cookie just run the function without any arguments, if you wish to check a specific cookie then enter the cookie as a string
    """
    try:
        if Cookie == None:
            session = robloxpy.CurrentCookie
        else:
            session = requests.session()
            CurrentCookie = {'.ROBLOSECURITY': Cookie}
            requests.utils.add_dict_to_cookiejar(session.cookies, CurrentCookie)
            Header = session.post('https://auth.roblox.com/')
            session.headers['x-csrf-token'] = Header.headers['x-csrf-token']
        response = session.get(MobileAPI + 'userinfo')
        try:
            Temp = response.json()['UserID']
            return True
        except (KeyError, JSONDecodeError):
            return False
    except TypeError:
        return False



import requests, sys
from . import user, Utils, errors
from typing import Union, Type, BinaryIO
from os import PathLike
from io import IOBase

robloxpy = sys.modules['robloxpy']

class ImageAsset():
    __slots__ = ('url')

    def __init__(self, url: str) -> None:
        self.url = url

    def __repr__(self):
        return self.url

    def __eq__(self, other: object) -> bool:
        if isinstance(other, ImageAsset):
            return self.url == other.url
        return False

    def read(self) -> bytes:
        """
        Returns the image in bytes.
        """
        return requests.get(self.url).content

    def save(self, fp: Union[BinaryIO, Type[PathLike]]) -> int:
        """
        Saves the image into a file-like object.
        """
        data = self.read()
        if isinstance(fp, IOBase) and fp.writable():
            return fp.write(data)
        else:
            with open(fp, 'wb') as f:
                return f.write(data)

class MarketAsset():
    __slots__ = ('id', 'name', 'description', 'creator', 'lowest_price', 'price', 'favorites')

    def __init__(self, id: int) -> None:
        self._update(id)

    def __repr__(self) -> str:
        return self.name

    def __eq__(self, other: object) -> bool:
        if isinstance(other, MarketAsset):
            return self.id == other.id
        return False

    def _update(self, id: int) -> None:
        if not robloxpy.CurrentCookie:
            raise errors.NoCookie
        try:
            data = robloxpy.CurrentCookie.post('https://catalog.roblox.com/v1/catalog/items/details', json={"items": [{"itemType": "Asset","id": id}]}).json()['data'][0]
        except IndexError:
            raise errors.InvalidId
        self.id = data['id']
        self.name = data['name']
        self.description = data['description']
        self.creator = user.User(data['creatorTargetId'])
        try:
            self.lowest_price = data['lowestPrice']
        except:
            self.lowest_price = None
        try:
            self.price = data['price']
        except:
            self.price = self.lowest_price or None
        self.favorites = data['favoriteCount']

    def thumbnail(self) -> Type[ImageAsset]:
        """
        Returns the assets thumbnail as an ImageAsset.
        """
        data = requests.get(f"{Utils.ThumnnailAPIV1}assets?assetIds={self.id}&format=Png&isCircular=true&size=700x700").json()['data'][0]
        return ImageAsset(data['imageUrl'])

    def buy(self) -> None:
        """
        purchases the asset.

        untested
        """
        response = robloxpy.CurrentCookie.post(f"https://economy.roblox.com/v1/purchases/products/{self.id}",data={"expectedCurrency":1,"expectedPrice":self.lowest_price,"expectedSellerId":None})


from . import group
from . import user
from . import Utils
from . import errors
from . import asset
from . import game
import requests
import sys

__version__ = "1.0.0.a1"

this = sys.modules[__name__]

this.CurrentCookie = None
this.RawCookie = None

def SetCookie(Cookie: str) -> None:
    """
    Set the current cookie for internal commands.
    """
    try:
        session = requests.session()
        CurrentCookie = {'.ROBLOSECURITY': Cookie}
        requests.utils.add_dict_to_cookiejar(session.cookies, CurrentCookie)
        Header = session.post('https://auth.roblox.com/')
        session.headers['x-csrf-token'] = Header.headers['x-csrf-token']
        session.headers["Origin"] = "https://www.roblox.com"
        session.headers["Referer"] = "https://www.roblox.com/"
    except:
        raise errors.InvalidCookie()
    if Utils.CheckCookie(Cookie) == False:
        raise errors.InvalidCookie()
    this.CurrentCookie = session
    this.RawCookie = Cookie



from setuptools import setup

with open('requirements.txt') as f:
  requirements = f.read().splitlines()

setup(name='robloxpy',
    version='1.0.0.a1',
    description='RobloxPy is an object oriented python wrapper for the roblox API.',
    author='KristanSmout & anytarseir67',
    author_email = '',
    url='https://github.com/KristanSmout/RobloxPyOfficial/tree/1.0',
    packages=['robloxpy'],
    install_requires=requirements,
    )


import requests
from . import Utils
from typing import List, Type, Union

User = None
PartialUser = None

class Group():
    __slots__ = ('id', 'name', 'description', 'owner', 'member_count', 'builders_club_only', 'public_entry')
    def __init__(self, id: int) -> None:
        self._update(id)

    def __repr__(self) -> str:
        return self.name

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Group):
            return self.id == other.id
        return False

    def _update(self, id: int) -> None:
        raw_json = requests.get(f"{Utils.GroupAPIV1}{id}").json()
        self.id = raw_json['id']
        self.name = raw_json['name']
        self.description = raw_json['description']
        if raw_json['owner'] != None:
            self.owner = User(raw_json['owner']['userId'])
        else:
            self.owner = None
        self.member_count = raw_json['memberCount']
        self.builders_club_only = raw_json['isBuildersClubOnly']
        self.public_entry = raw_json['publicEntryAllowed']

    @property
    def allies(self) -> List['Group']:
        """
        Returns the groups allies as a list of Group instances.
        """
        _allies = []
        data = requests.get(f"{Utils.GroupAPIV1}{self.id}/relationships/allies?model.startRowIndex=0&model.maxRows=100000").json()
        for group in data['relatedGroups']:
            _allies.append(Group(group['id']))
        return _allies

    @property
    def enemies(self) -> List['Group']:
        """
        Returns the groups enemies as a list of Group instances.
        """
        _enemies = []
        data = requests.get(f"{Utils.GroupAPIV1}{self.id}/relationships/enemies?model.startRowIndex=0&model.maxRows=100000").json()
        for group in data['relatedGroups']:
            _enemies.append(Group(group['id']))
        return _enemies

    def members(self, fetch: bool = False) -> Union[List['PartialUser'], List['User']]:
        """
        Returns a list of all members in the group. this method can be extraordinarily slow depending on group size.
        by default the members are not fetched and do not contain any attributes, this can be changed be setting fetch to True, be aware this has a high chance of being rate limited
        """
        Cursor = ""
        Done = False
        _members = []
        while(Done == False):
            response = requests.get(f"{Utils.GroupAPIV1}{self.id}/users?limit=100&sortOrder=Asc&cursor={Cursor}")
            _users = response.json()['data']
            if((response.json()['nextPageCursor'] == "null") or response.json()['nextPageCursor'] == None):
                Done = True
            else:
                Done = False
                Cursor = response.json()['nextPageCursor']
            for _user in _users:
                try:
                    if not fetch:
                        _members.append(PartialUser(_user['user']['userId']))
                        continue
                    _members.append(User(_user['user']['userId']))
                except:
                    pass
            if(response.json()['nextPageCursor'] == 'None'):
                Done = True
        return _members

def _get_user():
    global User
    global PartialUser
    from . import user as usr
    User = usr.User
    PartialUser = usr.PartialUser
_get_user()